* -------------------------------
* 	LPS Example
*
* 	Assembly Arrays
*
*	Language: MC68K-ASM1
*	Style: plain MC68000
* 	Version: Ref2-m68k
* -------------------------------

* Reference Plain MC68000 Solution 2

* Questo esempio mostra
* 1) Rappresentazione di array in ASM
* 2) Accesso a elementi di array mediante indirizzamento Indicizzato
* 3) Valutazione di espressioni con operandi in memoria

* Accesso agli elementi di un array
* Per accedere a l'elemento di indice K si deve calcolarne l'indirizzo.
* Rappresentando gli array come spiegato nella lezione "Organizzazione dei
* dati in memoria 3", nel caso di array1, in cui ogni elemento dell'array
* e' rappresentato come un byte, l'elemento a[K] e' rappresentato dal byte
* di indirizzo pari all'indirizzo dell'area di memoria che contiene l'array
* aumentata di K. Nel caso di array 2, in cui ogni elemento dell'array
* e' rappresentato come 4 byte, l'elemento a[K] e' rappresentato dal byte
* di indirizzo pari all'indirizzo dell'area di memoria che contiene l'array
* aumentata di 4*K.

* Valutazione di espressioni con operandi in memoria
* In MC68000 le istruzioni che svolgono operazioni aritmetico-logiche  possono
* avere un operando in memoria (ma non entrambi). Questa possibilita' viene
* sfruttata per ridurre il numero di istruzioni e di registri utilizzati.

* Questo esempio e' realizzato per l'assembler ASM-One V1.48

* Variabili:
; il tipo unsigned char viene memorizzato con byte
; il tipo long viene memorizzato con parole di 32 bit
	org	$4000

* array1
;	unsigned char array1[ 8 ] = { 9, 3, 0, 5, 1, 12, 19, 7 };
array1:	dc.b	9,3,0,5,1,12,19,7

* array2
;	long int array2[ N ] = { -5, 27, 39, -9, -58, 3, 67, -2, 4, 9 }, id1, id2;
array2:	dc.l	-5,27,39,-9,-58,3,67,-2,4,9

id1:	dc.w	6	; valore casuale da usare come input, simula scanf
id2:	dc.w	1	; valore casuale da usare come input, simula scanf

* Programma

	org	$2000
inizio_codice:

	* a1 address-of array1
	; a1 viene usato come registro base nei modi di indirizzamento
	; Indicizzato e Base-Indicizzato (si ricordi che tali modi
	; NON modificano i registri Base e Indice)
	move.l	#array1,a1

*	array1[ 3 ] += 8;
; Indirizzo di array1[ 3 ] = array1 + 3
	add.b	#8,3(a1)	; aggiunge 8 ad array1[ 3 ]

*	array1[ 5 ] = 3 * array1[ id1 ];
; Indirizzo di array1[ id1 ] = array1 + id1 ;
;	il calcolo precedente deve essere fatto dinamicamente
	move.w	id1,d1		; legge id1

; array1[ id1 ] e' un byte ma MULU richiede due operandi in formato word,
; quindi converte il byte in word, mediante zero-extension perche' e' il
; tipo di array1[ id1 ] e' unsigned char
	move.w	#0,d0		; per eseguire zero-extension, azzera la word
	move.b	(a1,d1.w),d0	; e poi vi copia il byte array1[ id1 ]
	mulu	#3,d0		; calcola 3 * array1[ id1 ]
	move.b	d0,5(a1)	; copia il risultato in array1[ 5 ]

	* a1 address-of array2
	; a1 viene usato come registro base nei modi di indirizzamento
	; Indicizzato e Base-Indicizzato (si ricordi che tali modi
	; NON modificano i registri Base e Indice)
	move.l	#array2,a1

*	array2[ 4 ] = 100 - array2[ 6 ];
; Indirizzo di array2[ 6 ] = array2 + 6*4 = array2 + 24
; Indirizzo di array2[ 4 ] = array2 + 4*4 = array2 + 16
	move.l	#100,d0		; copia la costante 100 in d0
	sub.l	6*4(a1),d0	; calcola in d0 il valore 100 - array2[ 6 ]
	move.l	d0,4*4(a1)	; copia il risultato in array2[ 4 ]

*	array2[ id2 ] -= array2[ 3 ];
; Indirizzo di array2[ 3 ] = array2 + 3*4 = array2 + 12
; Indirizzo di array2[ id2 ] = array2 + id2 * 4 ;
;	il calcolo precedente deve essere fatto dinamicamente
	move.l	3*4(a1),d0	; legge array2[ 3 ]

; calcola l'indirizzo di array2[ id2 ] ovvero array2 + id2 * 4
	move.w	id2,d1		; legge id2
	mulu	#4,d1		; calcola id2*4
;	OSSERVAZIONE: si potrebbe utilizzre uno shift a sinistra invece di MULU

	sub.l	d0,(a1,d1.w)	; calcola array2[ id2 ] - array2[ 3 ]

fine_codice:

	end

* Original Plain C Code

#include <stdio.h>

#define N 10

int main(void) {

	unsigned char array1[ 8 ] = { 9, 3, 0, 5, 1, 12, 19, 7 };
	long int array2[ N ] = { -5, 27, 39, -9, -58, 3, 67, -2, 4, 9 }, id1, id2;

	array1[ 3 ] += 8;

	scanf( "%ld", &id1 );

	array1[ 5 ] = 3 * array1[ id1 ];

	printf( "%hhx %hhx\n", array1[ 3 ], array1[ 5 ] );

	array2[ 4 ] = 100 - array2[ 6 ];

	scanf( "%ld", &id2 );
	
	array2[ id2 ] -= array2[ 3 ];

	printf( "%lx %lx\n", array2[ 4 ], array2[ id2 ] );

	return 0;
}
