# -------------------------------
# 	LPS Example
#
# 	Assembly Arrays
#
#	Language: MIPS32-MARS
#	Style: plain MIPS32
# 	Version: Ref2-mips
# -------------------------------

# Reference Plain MIPS32 Solution 2

# Questo esempio mostra
# 1) Rappresentazione di array in ASM
# 2) Accesso a elementi di array mediante indirizzamento Indicizzato
# 3) Valutazione di espressioni con operandi in memoria

# Accesso agli elementi di un array
# Per accedere a l'elemento di indice K si deve calcolarne l'indirizzo.
# Rappresentando gli array come spiegato nella lezione "Organizzazione dei
# dati in memoria 3", nel caso di array1, in cui ogni elemento dell'array
# e' rappresentato come un byte, l'elemento a[K] e' rappresentato dal byte
# di indirizzo pari all'indirizzo dell'area di memoria che contiene l'array
# aumentata di K. Nel caso di array 2, in cui ogni elemento dell'array
# e' rappresentato come 4 byte, l'elemento a[K] e' rappresentato dal byte
# di indirizzo pari all'indirizzo dell'area di memoria che contiene l'array
# aumentata di 4*K.

# Valutazione di espressioni con operandi in memoria
# In MIPS32 le istruzioni che svolgono operazioni aritmetico-logiche non 
# possono avere operandi in memoria. Quindi gli operandi in memoria vengono
# copiati in registri prima di valutare l'espressione e poi i risultati vengono
# copiati in memoria.

# Questo esempio e' realizzato per l'assembler MARS
# Per eseguire il programma si possono usare i comandi della GUI

# Variabili:
# il tipo unsigned char viene memorizzato con byte
# il tipo long viene memorizzato con parole di 32 bit
	.data	0x10010000

# array1
#	unsigned char array1[ 8 ] = { 9, 3, 0, 5, 1, 12, 19, 7 };
array1:	.byte	9,3,0,5,1,12,19,7

# array2
#	long int array2[ N ] = { -5, 27, 39, -9, -58, 3, 67, -2, 4, 9 }, id1, id2;
array2:	.word	-5,27,39,-9,-58,3,67,-2,4,9

id1:	.word	6	# valore da usare come input, simula scanf
id2:	.word	1	# valore da usare come input, simula scanf


# Programma

	.text
inizio_codice:	
	
	la	$s0,array1	# s0 address-of array1

#	array1[ 3 ] += 8;
# Indirizzo di array1[ 3 ] = array1 + 3
	lbu	$t0,3($s0)	# copia array1[ 3 ] in t0
	add	$t0,$t0,8	# aggiunge 8
	sb	$t0,3($s0)	# copia t0 in array1[ 3 ]
	
#	array1[ 5 ] = 3 * array1[ id1 ];
# Indirizzo di array1[ id1 ] = array1 + id1 ;
#	il calcolo precedente deve essere fatto dinamicamente
	lw	$t2,id1		# legge id1
	lbu	$t0,array1($t2)	# indirezione attraverso
				# indirizzamento indicizzato
	mul	$t0,$t0,3	# calcola 3 * array1[ id1 ]
	sb	$t0,5($s0)	# copia il risultato in array1[ 5 ]

	la	$s0,array2	# s0 address-of array2

#	array2[ 4 ] = 100 - array2[ 6 ];
# Indirizzo di array2[ 6 ] = array2 + 6*4 = array2 + 24
# Indirizzo di array2[ 4 ] = array2 + 4*4 = array2 + 16
	lw 	$t0,24($s0)	# legge array2[ 6 ]
	li	$t1,100		# copia 100 in t1
	sub	$t1,$t1,$t0	# calcola in t1 il valore 100 - array2[ 6 ]
	sw	$t1,16($s0)	# copia risultato in array2[ 4 ]

#	array2[ id2 ] -= array2[ 3 ];
# Indirizzo di array2[ 3 ] = array2 + 3*4 = array2 + 12
# Indirizzo di array2[ id2 ] = array2 + id2 * 4 ;
#	il calcolo precedente deve essere fatto dinamicamente
	lw	$t2,id2		# legge id2
	mulu	$t2,$t2,4	# calcola id2 * 4
	lw	$t0,array2($t2)	# t0 contiene array2[ id2 ]
	lw	$t1,12($s0)	# t2 contiene array2[ 3 ]
	sub	$t0,$t0,$t1	# calcola array2[ id2 ] - array2[ 3 ]
	sw	$t0,array2($t2)	# copia il risultato in array2[ id2 ]

fine_codice:	


# Original Plain C Code
#
# #include <stdio.h>
#
# #define N 10
#
# int main(void) {
#
#	unsigned char array1[ 8 ] = { 9, 3, 0, 5, 1, 12, 19, 7 };
#	long int array2[ N ] = { -5, 27, 39, -9, -58, 3, 67, -2, 4, 9 }, id1, id2;
#
#	array1[ 3 ] += 8;
#
#	scanf( "%ld", &id1 );
#
#	array1[ 5 ] = 3 * array1[ id1 ];
#
#	printf( "%hhx %hhx\n", array1[ 3 ], array1[ 5 ] );
#
#	array2[ 4 ] = 100 - array2[ 6 ];
#
#	scanf( "%ld", &id2 );
#	
#	array2[ id2 ] -= array2[ 3 ];
#
#	printf( "%lx %lx\n", array2[ 4 ], array2[ id2 ] );
#
#	return 0;
# }

